#lang racket
(require ffi/unsafe
         ffi/unsafe/define
         "curses.rkt")

(provide (all-defined-out))

(define-ffi-definer define-menu (ffi-lib "libmenu"))

(define _ITEM-pointer (_cpointer 'ITEM))
(define _MENU-pointer (_cpointer 'MENU))

#;
(define-menu new_item (_fun _bytes _bytes -> _ITEM-pointer))

;; The `new_item' function wants strings that live as long as the
;; item might be used --- and, implicitly, that do not move.
;; As a first step, allocate a fresh string,
;; and set up a finalizer to release the fresh string when the
;; original string is no longer live. Then, also make a finalizer-based
;; connection to the menu item produced by `new-item'.
(define _copied-string
  (make-ctype _pointer
              (lambda (s)
                (define bstr (string->bytes/utf-8 s))
                (define new (malloc (add1 (bytes-length bstr)) 'raw))
                (memcpy new bstr (add1 (bytes-length bstr)))
                (register-finalizer s (lambda (s) (free new)))
                new)
              (lambda (p)
                (cast p _pointer _string))))

(define-menu new_item (_fun (s1 : _copied-string) (s2 : _copied-string) 
                            -> (i : _ITEM-pointer)
                            -> (begin
                                 (register-finalizer i (lambda (i) s1))
                                 (register-finalizer i (lambda (i) s2))
                                 i)))
(define-menu new_menu (_fun (_list i (_or-null _ITEM-pointer)) -> _MENU-pointer))

(define-menu current_item (_fun _MENU-pointer -> _ITEM-pointer))

(define-menu set_menu_win (_fun _MENU-pointer _WINDOW-pointer -> _int))
(define-menu post_menu (_fun _MENU-pointer -> _int))

(define-menu menu_driver (_fun _MENU-pointer _int -> _int))

(define REQ_LEFT_ITEM           (+ KEY_MAX 1))
(define REQ_RIGHT_ITEM          (+ KEY_MAX 2))
(define REQ_UP_ITEM             (+ KEY_MAX 3))
(define REQ_DOWN_ITEM           (+ KEY_MAX 4))
(define REQ_CLEAR_PATTERN       (+ KEY_MAX 14))
